package com.kingmed.flutter_kingmed_version_tool;
import android.annotation.TargetApi;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.widget.Toast;

import androidx.core.content.FileProvider;
import androidx.annotation.NonNull;

import java.io.File;
import java.io.FileInputStream;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.HashMap;
import java.io.IOException;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import android.util.Log;

public class FlutterKingmedVersionToolPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  private MethodChannel channel;
  private Context context;
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_app_upgrade");
    channel.setMethodCallHandler(this);

  }

  @TargetApi(Build.VERSION_CODES.FROYO)
  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }else if(call.method.equals("getAppInfo")){
      getAppInfo(result);
    }else if(call.method.equals("getApkDownloadPath")){
      result.success(context.getExternalFilesDir("").getAbsolutePath());
    }
    else if(call.method.equals("install")){
      String path = call.argument("path");
      String md5=call.argument("md5");

      if (md5 != null && !md5.equals("null")&&!md5.equals("")) {
        Log.e("king",path);
        Log.e("king",getMd5(new File(path)));
        if (getMd5(new File(path)).equals(md5)) {
          install(path);
        } else {
          Toast.makeText(context, "文件损坏，请重新下载", Toast.LENGTH_LONG).show();
        }
      } else {
        //不校验MD5
        //Toast.makeText(context, "不校验MD5,直接安装", Toast.LENGTH_LONG).show();
        install(path);
      }

    }
    else {
      result.notImplemented();
    }
  }

  private void getAppInfo(Result result){
    HashMap hashMap=new HashMap<String,String>();
    PackageManager packageManager=context.getPackageManager();
    try {
      PackageInfo packageInfo=packageManager.getPackageInfo(context.getPackageName(),0);
      hashMap.put("versionName",packageInfo.versionName);
      hashMap.put("versionCode",packageInfo.versionCode+"");
      hashMap.put("packageName",packageInfo.packageName);
      result.success(hashMap);

    } catch (PackageManager.NameNotFoundException e) {
      e.printStackTrace();
    }
  }

  private void install(String path){

    Intent intent = new Intent(Intent.ACTION_VIEW);
    File file = new File(path);
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
      Uri apkUri = FileProvider.getUriForFile(context, context.getPackageName()+".fileprovider", file);
      intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
      intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
      intent.setDataAndType(apkUri, "application/vnd.android.package-archive");
    } else {
      intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
      Uri uri = Uri.fromFile(file);
      intent.setDataAndType(uri, "application/vnd.android.package-archive");
    }
    context.startActivity(intent);
  }

  /**
   * 获取文件的MD5值
   *
   * @param file
   * @return
   */
  private String getMd5(File file) {

    FileInputStream fis = null;
    try {
      MessageDigest digest = MessageDigest.getInstance("MD5");
      fis = new FileInputStream(file);
      byte[] buffer = new byte[8192];
      int len;
      while ((len = fis.read(buffer)) != -1) {
        digest.update(buffer, 0, len);
      }
      fis.close();
      return bytes2Hex(digest.digest());
    } catch (Exception e) {
      e.printStackTrace();
    }
    return "";
  }

  /**
   * 一个byte转为2个hex字符
   *
   * @param src byte数组
   * @return 16进制大写字符串
   */
  private static String bytes2Hex(byte[] src) {
    char[] res = new char[src.length << 1];
    final char hexDigits[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
    for (int i = 0, j = 0; i < src.length; i++) {
      res[j++] = hexDigits[src[i] >>> 4 & 0x0F];
      res[j++] = hexDigits[src[i] & 0x0F];
    }
    return new String(res);
  }
//  private String getMd5(File file) {
//    if (!file.isFile()) {
//      return null;
//    }
//    MessageDigest digest = null;
//    FileInputStream in = null;
//    byte buffer[] = new byte[1024];
//    int len;
//    try {
//      digest = MessageDigest.getInstance("MD5");
//      in = new FileInputStream(file);
//      while ((len = in.read(buffer, 0, 1024)) != -1) {
//        digest.update(buffer, 0, len);
//      }
//      in.close();
//    } catch (Exception e) {
//      e.printStackTrace();
//      return null;
//    }
//    BigInteger bigInt = new BigInteger(1, digest.digest());
//    //Toast.makeText(context, bigInt.toString(16), Toast.LENGTH_LONG).show();
//    return bigInt.toString(16);
//  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding activityPluginBinding) {
    context=activityPluginBinding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding activityPluginBinding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }
}
