part of flutter_kingmed_version_tool;


class AppUpgrade {

  static appUpgrade(
    BuildContext context,
    Future<AppUpgradeInfo> future, {
    required TextStyle titleStyle,
    required TextStyle contentStyle,
    required TextStyle cancelTextStyle,
    required TextStyle okTextStyle,
    required String cancelText,
    required String okText,
    required Color progressBarColor,
    double borderRadius = 20.0,
    required String iosAppId,
    required AppMarketInfo appMarketInfo,
    required VoidCallback onCancel,
    required VoidCallback onOk,
    required DownloadProgressCallback downloadProgress,
    required DownloadStatusChangeCallback downloadStatusChange,
  }) {
    future.then((AppUpgradeInfo appUpgradeInfo) {
      if (appUpgradeInfo != null && appUpgradeInfo.title != null) {
        _showUpgradeDialog(
            context, appUpgradeInfo.title, appUpgradeInfo.contents,
            apkDownloadUrl: appUpgradeInfo.apkDownloadUrl,
            force: appUpgradeInfo.force,
            titleStyle: titleStyle,
            contentStyle: contentStyle,
            cancelText: cancelText,
            cancelTextStyle: cancelTextStyle,
            okText: okText,
            okTextStyle: okTextStyle,
            borderRadius: borderRadius,
            progressBarColor: progressBarColor,
            iosAppId: iosAppId,
            md5: appUpgradeInfo.md5,
            appMarketInfo: appMarketInfo,
            onCancel: onCancel,
            onOk: onOk,
            downloadProgress: downloadProgress,
            downloadStatusChange: downloadStatusChange);
      }
    }).catchError((onError) {
        print('$onError');
    });
  }

  static _showUpgradeDialog(
    BuildContext context,
    String title,
    String contents, {
    required String apkDownloadUrl,
    bool force = false,
    required TextStyle titleStyle,
    required TextStyle contentStyle,
    required String cancelText,
    required TextStyle cancelTextStyle,
    required String okText,
    required TextStyle okTextStyle,
    List<Color>? okBackgroundColors,
    required Color progressBarColor,
    double borderRadius = 20.0,
    String? md5,
    required String iosAppId,
    required AppMarketInfo appMarketInfo,
    required VoidCallback onCancel,
    required VoidCallback onOk,
    required DownloadProgressCallback downloadProgress,
    required DownloadStatusChangeCallback downloadStatusChange,
  }) {
    showDialog(
        context: context,
        barrierDismissible: false,
        builder: (context) {
          return WillPopScope(
            onWillPop: () async {
              return false;
            },
            child: Dialog(
                shape: RoundedRectangleBorder(
                    borderRadius:BorderRadius.all(Radius.circular(borderRadius))),
                child: SimpleAppUpgradeWidget(
                  title: title,
                  titleStyle: titleStyle,
                  contents: contents,
                  contentStyle: contentStyle,
                  cancelText: cancelText,
                  cancelTextStyle: cancelTextStyle,
                  okText: okText,
                  okTextStyle: okTextStyle,

                  progressBarColor: progressBarColor,
                  borderRadius: borderRadius,
                  downloadUrl: apkDownloadUrl,
                  force: force,
                  md5: md5,
                  iosAppId: iosAppId,
                  appMarketInfo: appMarketInfo,
                    onCancel: onCancel,
                    onOk: onOk,
                    downloadProgress: downloadProgress,
                    downloadStatusChange: downloadStatusChange, progressBar: LinearProgressIndicator(),
                )),
          );
        });
  }
}

class AppInfo {
  AppInfo({required this.versionName, required this.versionCode, required this.packageName});
  String versionName;
  String versionCode;
  String packageName;

}

class AppUpgradeInfo {
  AppUpgradeInfo(
      {required this.title,
      required this.contents,
      required this.apkDownloadUrl,
      this.force = false,
      this.md5
      });

  final String title;

  String contents;

  final String apkDownloadUrl;

  final bool force;

  //留空不校验MD5
  String ? md5;

}


// 下载进度回调

typedef DownloadProgressCallback = Function(int count, int total);

// 下载状态变化回调

typedef DownloadStatusChangeCallback = Function(DownloadStatus downloadStatus,{dynamic error});
