part of flutter_kingmed_version_tool;

class SimpleAppUpgradeWidget extends StatefulWidget {
  const SimpleAppUpgradeWidget(
      {required this.title,
        this.titleStyle=const TextStyle(),
      required this.contents,
      required this.contentStyle,
         this.cancelText='以后再说',
       required  this.cancelTextStyle,
         this.okText='确定升级',
        required this.okTextStyle,
      required this.progressBar,
      required this.progressBarColor,
      this.borderRadius = 10,
        required this.downloadUrl,
        this.force = false,
        required this.iosAppId,
        required this.appMarketInfo,
        this.onCancel,
        this.onOk,
        this.downloadProgress,
        this.downloadStatusChange,
        this.md5
      });

  final String title;
  final TextStyle titleStyle;
  final String contents;
  final TextStyle contentStyle;
  final Widget progressBar;
  final Color progressBarColor;
  final String okText;
  final TextStyle okTextStyle;
  final String cancelText;
  final TextStyle cancelTextStyle;
  final String downloadUrl;
  final double borderRadius;
  final bool force;
  final String iosAppId;
  final String ? md5;
  final AppMarketInfo appMarketInfo;

  final VoidCallback ? onCancel;
  final VoidCallback ? onOk;
  final DownloadProgressCallback? downloadProgress;
  final DownloadStatusChangeCallback? downloadStatusChange;

  @override
  State<StatefulWidget> createState() => _SimpleAppUpgradeWidget();
}

class _SimpleAppUpgradeWidget extends State<SimpleAppUpgradeWidget> {
  static final String _downloadApkName = 'temp.apk';
  double _downloadProgress = 0.0;
  DownloadStatus _downloadStatus = DownloadStatus.none;
  String okTitle='马上升级';
  @override
  Widget build(BuildContext context) {
    return Container(
      child:_buildInfoWidget(context),
    );
  }

  Widget buildProcessBar(){
     return  Row(
         children: [
           SizedBox(width: 12),
           Expanded(child: LinearProgressIndicator(value: _downloadProgress),),
           SizedBox(width: 12),
           Text((_downloadProgress*100).toInt().toString()+'%'),
           SizedBox(width: 12),
         ],
     );
  }

  ///
  /// 信息展示widget
  ///
  Widget _buildInfoWidget(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        //标题
        _buildTitle(),
        //更新信息
        _buildAppInfo(),

        SizedBox(height: 10,),

        _downloadProgress > 0? buildProcessBar():SizedBox(),

        SizedBox(height: 10,),

        //操作按钮
        _buildAction(),

      ],
    );
  }


  _buildTitle() {
    return Center(
       child: Padding(
            padding: EdgeInsets.only(top: 20, bottom: 30),
            child: Text(widget.title,style: widget.titleStyle))
    );
  }

  _buildAppInfo() {
    return Container(
        padding: EdgeInsets.only(left: 15, right: 15, bottom: 30),
        height: 200,
        child: Text(widget.contents,style: widget.contentStyle,));
  }

  _buildAction() {
    return Column(
      children: <Widget>[
        Divider(
          height: 1,
          color: Colors.grey,
        ),
        Row(
          children: <Widget>[
            _buildBackGroundLeftButton(),
            Expanded(
              child: _buildBackGroundButton(),
            ),

          ],
        ),
      ],
    );
  }

  Widget _buildBackGroundLeftButton() {
    if (_downloadProgress > 0||widget.force) {
      //显示后台更新接口
      return SizedBox();
    } else {
      return Expanded(child: _buildCancelActionButton());
    }
  }

  Widget _buildBackGroundButton() {
    if (_downloadProgress > 0) {
      //显示后台更新接口
      if (widget.force) {
        return SizedBox();
      } else {
        return _buildBgUpdateButton();
      }
    } else {
      return _buildOkActionButton();
    }
  }


  _buildCancelActionButton() {
    return Ink(
      decoration: BoxDecoration(
          borderRadius: BorderRadius.only(
              bottomLeft: Radius.circular(widget.borderRadius))),
      child: InkWell(
          borderRadius: BorderRadius.only(
              bottomLeft: Radius.circular(widget.borderRadius)),
          child: Container(
            height: 45,
            alignment: Alignment.center,
            child: Text(widget.cancelText,style: widget.cancelTextStyle),
          ),
          onTap: () {
            widget.onCancel?.call();
            Navigator.of(context).pop();
          }),
    );
  }


  _buildBgUpdateButton() {

    var borderRadius =BorderRadius.only( bottomRight: Radius.circular(widget.borderRadius),bottomLeft: Radius.circular(widget.borderRadius));

    return Ink(
      decoration: BoxDecoration(
          color: new Color(0xff00C4AF),
          borderRadius: borderRadius),
      child: InkWell(
        borderRadius: borderRadius,
        child: Container(
          height: 45,
          alignment: Alignment.center,
          child: Text('取消更新',
              style: TextStyle(color: Colors.white)),
        ),
        onTap: () {
           Navigator.of(context).pop();
        },
      ),
    );
  }


  _buildOkActionButton() {
    var borderRadius =
        BorderRadius.only(bottomRight: Radius.circular(widget.borderRadius));
    if (widget.force) {
      borderRadius = BorderRadius.only(
          bottomRight: Radius.circular(widget.borderRadius),
          bottomLeft: Radius.circular(widget.borderRadius));
    }

    return Ink(
      decoration: BoxDecoration(
          color: new Color(0xff00C4AF),
          borderRadius: borderRadius),
      child: InkWell(
        borderRadius: borderRadius,

        child: Container(
          height: 45,
          alignment: Alignment.center,

          child: Text(okTitle,style: widget.okTextStyle),
        ),
        onTap: () {
          _clickOk();
        },
      ),
    );
  }

  ///
  /// 点击确定按钮
  ///
  _clickOk() async {
    widget.onOk?.call();
    if (Platform.isIOS) {
      //ios 需要跳转到app store更新，原生实现
      FlutterUpgrade.toAppStore(widget.iosAppId);
      return;
    }
    if (widget.downloadUrl == null || widget.downloadUrl.isEmpty) {
      //没有下载地址，跳转到第三方渠道更新，原生实现
      FlutterUpgrade.toMarket(appMarketInfo: widget.appMarketInfo);
      return;
    }

    String path = await FlutterUpgrade.apkDownloadPath;
    
    if(widget.force){
      _downloadApk(widget.downloadUrl, '$path/$_downloadApkName');
    }else{
      // 测试测试测试
      Navigator.of(context).pop();
      new DownLoadOverlay(context, widget.downloadProgress, widget.downloadStatusChange, widget.md5, widget.downloadUrl,'$path/$_downloadApkName').show();
    }

  }

  ///
  /// 下载apk包
  ///
  _downloadApk(String url, String path) async {
    if (_downloadStatus == DownloadStatus.start ||
        _downloadStatus == DownloadStatus.downloading ||
        _downloadStatus == DownloadStatus.done) {
      print('当前下载状态：$_downloadStatus,不能重复下载。');
      return;
    }

    _updateDownloadStatus(DownloadStatus.start);

    try {

      var dio = Dio();
      await dio.download(url, path, onReceiveProgress: (int count, int total) {
        if (total == -1) {
          _downloadProgress = 0.01;
        } else {
          widget.downloadProgress?.call(count, total);
          _downloadProgress = count / total.toDouble();
        }
        setState(() {});
        if (_downloadProgress == 1) {
          //下载完成，跳转到程序安装界面
          _updateDownloadStatus(DownloadStatus.done);

          if(!widget.force){
            Navigator.pop(context);
          }

          FlutterUpgrade.installAppForAndroid(path,widget.md5.toString());
        }
      });
    } catch (e) {

      print('下载出错:'+'$e');
      okTitle='下载出错-重试';
      setState(() {

      });
      _downloadProgress = 0;
      _updateDownloadStatus(DownloadStatus.error,error: e);
    }
  }

  _updateDownloadStatus(DownloadStatus downloadStatus, {dynamic error}) {
    _downloadStatus = downloadStatus;
    widget.downloadStatusChange?.call(_downloadStatus, error: error);
  }
}
